@@ -10,10 +10,11 @@ use clap::{
 
 use printnanny_services::config::{ PrintNannyConfig};
 use printnanny_services::janus::{ JanusAdminEndpoint, janus_admin_api_call };
-use printnanny_services::mqtt::{ MQTTWorker, MqttAction };
+use printnanny_services::mqtt::{ MQTTWorker };
+use printnanny_services::remote;
 use printnanny_cli::device::{DeviceCmd, DeviceAction };
 use printnanny_cli::config::{ConfigAction};
-use printnanny_cli::remote;
+use printnanny_api_client::models;
 
 #[tokio::main]
 async fn main() -> Result<()> {
@@ -101,17 +102,22 @@ async fn main() -> Result<()> {
                 .takes_value(true)
             ))
         // mqtt <subscribe|publish>
-        .subcommand(App::new("mqtt")
+        .subcommand(App::new("event")
             .author(crate_authors!())
             .about(crate_description!())
             .version(crate_version!())
             .about("Interact with MQTT pub/sub service")
-            .setting(AppSettings::ArgRequiredElseHelp)
-            .arg(Arg::new("action")
-                .possible_values(MqttAction::possible_values())
-                .ignore_case(true)
+            .setting(AppSettings::SubcommandRequiredElseHelp)
+            .subcommand(
+                App::new("publish")
+                .arg(Arg::new("data")
+                    .short('d')
+                    .long("data")
+                    .takes_value(true)
+            ))
+            .subcommand(
+                App::new("subscribe")
             ))
-
         .subcommand(App::new("remote")
             .author(crate_authors!())
             .about(crate_description!())
@@ -148,16 +154,24 @@ async fn main() -> Result<()> {
     };
 
     match app_m.subcommand() {
-        Some(("mqtt", sub_m)) => {
-            let action: MqttAction = sub_m.value_of_t("action").unwrap_or_else(|e| e.exit());
-            match action {
-                MqttAction::Subscribe => {
+        Some(("event", sub_m)) => {
+            match sub_m.subcommand() {
+                Some(("subscribe", _event_m)) => {
                     let worker = MQTTWorker::new(
                         config,
                     ).await?;
-                    worker.run().await?;
+                    worker.subscribe().await?;
+                }
+                Some(("publish", event_m)) => {
+                    let worker = MQTTWorker::new(
+                        config,
+                    ).await?;
+                    let data = event_m.value_of("data").expect("Expected --data argument passed");
+                    let event: models::PolymorphicEventRequest = serde_json::from_str(data).expect("Failed to deserialize event data");
+                    let value: serde_json::Value = serde_json::to_value(event)?;
+                    worker.publish(value).await?;
                 },
-                MqttAction::Publish => unimplemented!("mqtt publish is not implemented yet")
+                _ => panic!("Expected publish|subscribe subcommand")
             }
         },
         Some(("config", _)) => {
@@ -180,10 +194,11 @@ async fn main() -> Result<()> {
 
         },
 
-        Some(("remoter", sub_m)) => {
+        Some(("remote", sub_m)) => {
             let dryrun = sub_m.is_present("dryrun");
             let json_str = sub_m.value_of("event").expect("--event argument is required");
-            remote::handle_event(json_str, config, dryrun)?;
+            let event: models::PolymorphicEvent = serde_json::from_str(json_str).expect("Failed to deserialize event data");
+            remote::handle_event(event, config, dryrun)?;
         }
 
         Some(("system-update", _sub_m)) => {
